#!/bin/bash
#

# buildGCC.sh <product_dir> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
      flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch

}

die()
{
  echo "$1"; exit 2
}

unpack_and_patch()
{
    local tardest=${blddir}/src
    mkdir -p ${tardest}

    # unpack
    gcctar=${tardir}/gcc-${origpkgdotver}.tar.bz2
    if [[ -d "${tardest}/gcc-${origpkgdotver}" ]]; then
      echo "GCC source already exists."
    else
      tar -xf ${gcctar} -C ${blddir}/src || die "Unable to untar gcc source into ${blddir}/src"
    fi
    tardest="${tardest}/gcc-${origpkgdotver}"
    for tarfile in "${tardir}/"*.tar.*; do
      [[ "${tarfile}" == "${gcctar}" ]] && continue
      tarfile=$(basename "${tarfile}")
      subpkg=${tarfile%%-*}
      subpkgdir=${tarfile%.tar*}
      subpkgdir=${subpkgdir%%[a-z]}
      if [[ -d "${tardest}/${subpkg}" ]]; then
        echo "${subpkg} source already exists in ${tardest}"
      else
        rm -rf "${tardest}/${subpkgdir}"
        { tar -xf "${tardir}/${tarfile}" -C "${tardest}" && mv "${tardest}/${subpkgdir}" "${tardest}/${subpkg}"; } \
         || die "Unable to untar ${subpkg} source into ${tardest}"
      fi
      if [[ -f "${patchdir}/${subpkgdir}.patch" ]]; then
        ( cd "${tardest}/${subpkgdir}" && patch -b -p0 --verbose < "${patchdir}/${subpkgdir}.patch" || die "Unable to apply patch ${subpkgdir}.patch to ${subpkg}" )
      fi
    done
    flvr1=`ups flavor -1`
    if [ ${flvr1} = "Darwin" ]
    then
	set -x
	cd ${blddir}/src
	perl -pi.bak -e 's/uname -p/uname -m/go;' */config.guess */configfsf.guess
	set +x
    fi
}

check_for_file() {
  local dir
  local verbose
  local x
  while getopts :dvx OPT; do
    case $OPT in
      d)
        dir=1
        ;;
      v)
        verbose=1
        ;;
      x)
        x=1
        ;;
      *)
        echo "INTERNAL ERROR in call to check_for_file." 1>&2
        exit 2
    esac
  done
  shift $(( OPTIND - 1 ))
  OPTIND=1
  local opt
  local entity
  if (( ${x:-0} )); then
    opt=-x
    entity=executable
  elif (( ${dir:-0} )); then
    opt=-d
    entity=directory
  else
    opt=-f
    entity=file
  fi
  eval [[ ${opt} "${1}" ]]
  (( status = $? ))
  if (( ${status} && ${verbose:-0} )); then
    echo "Expected ${entity} \"${1}\" not found as specified." 1>&2
  fi
  return ${status}
}

gcc_built() {
  local verbose
  while getopts :v OPT; do
    case $OPT in
      v)
        verbose=-v
        ;;
      *)
        echo "INTERNAL ERROR in call to gcc_built" 1>&2
        exit 2
    esac
  done
  shift $(( OPTIND - 1 ))
  OPTIND=1
  local ok
  (( ok = 1 ))
  check_for_file ${verbose} "${pkgdir}/lib64/libstdc++.${shext}"
  (( ok &= $? ))
  check_for_file ${verbose} "${pkgdir}/lib64/libgfortran.${shext}"
  (( ok &= $? ))
  check_for_file ${verbose} -x "${pkgdir}/bin/gcc"
  (( ok &= $? ))
  check_for_file ${verbose} -x "${pkgdir}/bin/g++"
  (( ok &= $? ))
  check_for_file ${verbose} -x "${pkgdir}/bin/gfortran"
  (( ok &= $? ))
  check_for_file ${verbose} -d "${pkgdir}/include/c++"
  (( ok &= $? ))
  return $(( ok == 1 ))
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}
sourcetar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=gcc
origpkgver=v6_2_0
pkgver=${origpkgver}
ssibuildshims_version=v0_10_01
origpkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

get_platform_name

if [ ${OS1} = "Darwin" ]
then
  echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"
  export DYLD_LIBRARY_PATH=${pkgdir}/lib64:${pkgdir}/lib
  shext=dylib
else
  echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
  export LD_LIBRARY_PATH=${pkgdir}/lib64:${pkgdir}/lib
  shext=so
fi

if [ "${maketar}" = "tar" ] && gcc_built
then
    ups exist ${package} ${pkgver} || ${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_QUAL
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL
    exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi


unpack_and_patch

# set the build environment
# set CC and CXX
export CC=gcc
export CXX=g++
[[ "${OS1}" == Darwin ]] && export LDFLAGS=-Wl,-headerpad_max_install_names
num_cores=$(${SSIBUILDSHIMS_DIR}/bin/ncores)
# remove /usr/local/bin and . from path
PATH=`dropit -e /usr/local/bin .`
# remove /usr/local/X from LD_LIBRARY PATH and DYLD_LIBRARY_PATH
LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+$(dropit -a -p "$LD_LIBRARY_PATH" /usr/local/)}
DYLD_LIBRARY_PATH=${DYLD_LIBRARY_PATH:+$(dropit -a -p "$DYLD_LIBRARY_PATH" /usr/local/)}
# make sure libraries are in the path
echo "COMPILER_PATH   $COMPILER_PATH"
export PATH=${pkgdir}/bin:${PATH}
export COMPILER_PATH=${pkgdir}/bin
# need proper environment
echo ""
echo "CC $CC CXX $CXX "
echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
[[ -n "${LDFLAGS}" ]] && echo "LDFLAGS ${LDFLAGS}"
echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"
echo "COMPILER_PATH   $COMPILER_PATH"
# build gcc
if gcc_built
then
    echo "gcc has already been built"
else
    set -x
    cd ${blddir}/src/gcc-*
    ./configure --prefix=${pkgdir} \
                --disable-libgcj \
                --enable-threads=posix \
                --enable-__cxa_atexit \
                --enable-languages=c,c++,fortran,lto \
                --with-system-zlib \
                --enable-libstdcxx-time=rt \
                --enable-stage1-checking=yes \
                --enable-checking=release \
                --enable-lto \
                --enable-plugin \
     || \
     die "ERROR: Unable to configure gcc."
    if [[ "${OS1}" == "Darwin" ]]; then
        # profiledbootstrap not reliable on OS X with clang compiling first stage.
        target=bootstrap
    else
        # Reputed to produce a compiler around 7% faster according to
        # benchmarks.
        target=profiledbootstrap
    fi
    make ${target} -j "${num_cores}" || die "ERROR: Unable to make gcc."
    make install || die "ERROR: Unable to install gcc."
    set +x
fi
if ! gcc_built -v
then
   echo "ERROR: the gcc build has failed"
   echo "       Something is very wrong"
   exit 1
fi

if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# Remove .la files.
find "${pkgdir}" -name '*.la' | xargs rm -vf

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_QUAL

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -z ${product_dir}:${PRODUCTS} || die "Unable to set up newly-built product"
echo "${package} is installed at ${GCC_FQ_DIR}"


# this must be last because it will exit
if [ "${maketar}" = "tar" ] && gcc_built
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL
fi

exit 0
